// Copyright (C) 2004 Johan Srensen
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/* $Id: MyWebKitController.m 38 2004-09-15 06:59:20Z  $ */

#import "MyWebKitController.h"

@implementation MyWebKitController

- (IBAction)fetch:(id)sender
{
	//NSLog(@"selectedWidth: %i", [selectedWidth intValue]);
	//NSLog(@"selectedHeight: %i", [selectedHeight intValue]);
	int initWidth = [selectedWidth intValue];
//	int initWidth = 800;
	int initHeight = [selectedHeight intValue];
//	int initHeight = 600;
	// create a new window
	win = [[NSWindow alloc] 
			initWithContentRect:NSMakeRect(-16000.0, -16000.0, initWidth, initHeight)
			styleMask: NSBorderlessWindowMask 
			backing:NSBackingStoreRetained 
			defer:NO];
	
	// create a WebView object
	webView = [[WebView alloc] initWithFrame:NSMakeRect(-16000.0, -16000.0, initWidth, initHeight)];

	// Create the delegates so we can play with it
	[webView setFrameLoadDelegate:self];
    [webView setResourceLoadDelegate:self];
	
	// remove scrollbars, so the content ijhkls x wide and not x-15
	[[[webView mainFrame] frameView] setAllowsScrolling:NO];
	//add the webview to the windowkjl
	[win setContentView:webView];
	
	// Register notifications
	NSNotificationCenter*   center;
    center = [NSNotificationCenter defaultCenter];
    [center addObserver:self 
            selector:@selector(webViewProgressStarted:) 
            name:WebViewProgressStartedNotification 
            object:nil];
    [center addObserver:self 
            selector:@selector(webViewProgressEstimateChanged:) 
            name:WebViewProgressEstimateChangedNotification 
            object:nil];
    [center addObserver:self 
            selector:@selector(webViewProgressFinished:) 
            name:WebViewProgressFinishedNotification 
            object:nil];		
	
	// Load the requested URL
	[[webView mainFrame] loadRequest:[NSURLRequest 
		requestWithURL:[NSURL URLWithString:[urlText stringValue]]]];
 
}

- (void)webView:(WebView *)sender 
	didFailProvisionalLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
{
	// what happens of something goes wrong while loading url (dns error etc)
	NSLog(@"didFailProvisionalLoadWithError: Something went wrong!");
	NSAlert *alert = [[NSAlert alloc] init];
	[alert addButtonWithTitle:@"OK"];
	[alert setMessageText:@"Something went wrong!"];
	[alert setInformativeText:@"The address you specified may be incorrect or the site may have DNS issues"];
	[alert setAlertStyle:NSWarningAlertStyle];	
	
	//display the alert sheet
	[alert beginSheetModalForWindow:[NSApp mainWindow] 
							modalDelegate:self 
							didEndSelector:@selector(alertDidEnd:returnCode:contextInfo:) 
							contextInfo:nil];
	
}
- (void)alertDidEnd:(NSAlert *)alert returnCode:(int)returnCode 
	contextInfo:(void *)contextInfo 
{
	// we don't really need to do anything here after the user has clicked 'ok' to the alert
	// other than releasing the object of course
	[alert release];
}

- (void)webView:(WebView *)sender
		didStartProvisionalLoadForFrame:(WebFrame *)frame
{
	//NSLog(@"didStartProvisionalLoadForFrame=true, updating pageprogress indicator");
	//NSLog(@"webView estimatedProgress: %f", [webView estimatedProgress]);
}

- (void)webView:(WebView*)sender 
                didFinishLoadForFrame:(WebFrame*)frame
{
	if (frame == [sender mainFrame]) {		
		// set the size to the natural contents of the page
		viewport = [[[webView mainFrame] frameView] documentView]; // width/height of html page
		//viewport = [[webView mainFrame] frameView];
		[[viewport window]  display];
		[[viewport window] setContentSize:[viewport bounds].size];
		[viewport setFrame: [viewport bounds]];
		
		//NSLog(@"viewport size - w: %f h: %f", [viewport bounds].size.width, [viewport bounds].size.height);
		
		// take the screenshot
		[webView lockFocus];  // lock the view
		//bitmap = [[NSBitmapImageRep alloc] initWithFocusedViewRect:[webView frame]];
		bitmap = [[NSBitmapImageRep alloc] initWithFocusedViewRect:[viewport bounds]];
		[webView unlockFocus]; //unlock the view

		dispimage = [[NSImage alloc] initWithData:[bitmap TIFFRepresentation]]; 
		
		// Display the image
		[imageview setImage:(NSImage *)dispimage];
		[saveButton setEnabled:YES];	
		
		// Release the objects we allocated
		[win release];	
		[webView release];	
	}
}

//--------------------------------------------------------------//
// The "save image as PNG" sheet
//--------------------------------------------------------------//
- (IBAction)saveDocumentAs:(id)sender
{
	// figure out a suitable string for the filename
	NSMutableString* suggestedFileName = [[[urlText stringValue] substringFromIndex:7] mutableCopy];
	// Replace some ugly strings
	[suggestedFileName replaceOccurrencesOfString:@"/" withString:@"-"
							options:NSLiteralSearch
							range:NSMakeRange(0, [suggestedFileName length])];
	[suggestedFileName replaceOccurrencesOfString:@"~" withString:@"-"
							options:NSLiteralSearch
							range:NSMakeRange(0, [suggestedFileName length])];
	[suggestedFileName replaceOccurrencesOfString:@"@" withString:@"-"
							options:NSLiteralSearch
							range:NSMakeRange(0, [suggestedFileName length])];

	
	//NSLog(@"suggested file: %@", suggestedFileName);
	
	// TODO: add setCanSelectHiddenExtension:YES to the savepanel
	NSSavePanel* sp = [NSSavePanel savePanel];
	[sp setCanSelectHiddenExtension:YES];
	[sp setRequiredFileType:@"png"];	
	[sp
		beginSheetForDirectory:NSHomeDirectory()  // seems like a reasonable place to start..
		file:suggestedFileName   // default filename
		modalForWindow:[NSApp mainWindow]  //Run as a sheet, attached to "window"
		modalDelegate:self   // The save panel should call me when it's done..
		didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:) // ... using this selector
		contextInfo:NULL];
}

- (void)savePanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
	if (returnCode == NSOKButton)  // The user hit "save".
		[[bitmap representationUsingType:NSPNGFileType properties:nil] 
			//writeToFile:[NSString stringWithFormat:@"%@.%@", [sheet filename], @"png"]
			writeToFile:[NSString stringWithFormat:@"%@", [sheet filename]]
			atomically:YES];
}
//--------------------------------------------------------------//
// WebView notification
//--------------------------------------------------------------//

- (void)webViewProgressStarted:(NSNotification*)notification
{
    //NSLog(@"webViewProgressStarted");
	//reset the progress indicator
	[pageloadprogress setDoubleValue:0.0];
}

- (void)webViewProgressEstimateChanged:(NSNotification*)notification
{
	//NSLog(@"webViewProgressEstimateChanged to: %f", [webView estimatedProgress]);    
	[pageloadprogress displayIfNeeded];
	//[pageloadprogress setDoubleValue:[webView estimatedProgress]];
	[pageloadprogress setDoubleValue:[webView estimatedProgress]*100];
}

- (void)webViewProgressFinished:(NSNotification*)notification
{    
	//NSLog(@"webViewProgressFinished");
}



@end
